#include "defs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>


void
usage (char *prg)
{
  printf ("Usage: %s [OPTIONS]... [FILE]\n", prg);
  printf ("\nOptions:\n");
  printf ("  -f,  --factor		ifps/ofps\n");
  printf ("  -m,  --move		+/- seconds\n");
  printf ("  -o,  --output-file	output file name\n");
  printf ("  -t,  --output-type	output file type (srt, sub, txt)\n");
  printf ("       --ifps		input file frames per second\n");
  printf ("       --ofps		output file frames per second\n");
}


int
main (int argc, char *argv[])
{
  FILE *fp, *fpout;
  char subtype, outtype, outtflag, ifpsflag, ofpsflag, sub_begin;
  char *line, *subfile, *outfile;
  unsigned long cnt, tlstart, tbl, tel;
  double tdstart, tbd, ted;
  double fps, ofps, move, factor;

  int i, c;
  int digit_optind;

  fps = 25.0;
  ofps = 25.0;
  move = 0;
  factor = 1;
  ifpsflag = 0;
  ofpsflag = 0;
  outtflag = 0;
  outfile = NULL;

  digit_optind = 0;

  while (1)
    {
      int this_option_optind = optind ? optind : 1;
      int option_index = 0;
      static struct option long_options[] = {
	{"factor", 1, 0, 'f'},
	{"move", 1, 0, 'm'},
	{"output-file", 1, 0, 'o'},
	{"output-type", 1, 0, 't'},
	{"ifps", 1, 0, 0},
	{"ofps", 1, 0, 0},
	{0, 0, 0, 0}
      };

      c = getopt_long (argc, argv, "f:m:o:t:", long_options, &option_index);

      if (c == -1)
	break;

      switch (c)
	{
	case 0:
	  if (optarg)
	    {
	      if (option_index == 4)
		{
		  fps = atof (optarg);
		  ifpsflag = 1;
		}
	      else if (option_index == 5)
		{
		  ofps = atof (optarg);
		  ofpsflag = 1;
		}
	    }
	  break;

	case 'f':
	  factor = atof (optarg);
	  break;

	case 'm':
	  move = atof (optarg);
	  break;

	case 'o':
	  outfile = (char *) malloc ((strlen (optarg) + 1) * sizeof (char));
	  strcpy (outfile, optarg);
	  break;

	case 't':
	  outtflag = 1;
	  if (!strcmp (optarg, "txt"))
	    outtype = 1;
	  else if (!strcmp (optarg, "srt"))
	    outtype = 2;
	  else if (!strcmp (optarg, "sub"))
	    outtype = 3;
	  else
	    {
	      fprintf (stderr, "%s: Unknown output subtitle type\n", optarg);
	      exit (-3);
	    }
	  break;

	case '?':
	  usage (argv[0]);
	  exit (-63);
	  break;

	default:
	  printf ("?? getopt returned character code 0%o ??\n", c);
	}
    }

  if (optind < argc)
    {
      subfile = argv[optind];
      while (optind < argc)
	argv[optind++];
    }
  else
    {
      usage (argv[0]);
      exit (-1);
    }

  if (outfile != NULL)
    {
      if ((fpout = fopen (outfile, "wb")) == NULL)
	{
	  perror (argv[argc - 1]);
	  exit (-2);
	}
    }
  else
    fpout = stdout;

  fprintf (stderr, "Detecting subtitle type ...\n");
  subtype = sub_detect_type (subfile);
  if (subtype == 0)
    {
      fprintf (stderr, "%s: Unknown subtitle type\n", subfile);
      exit (-3);
    }

  fprintf (stderr, "Detected.\n");
  fprintf (stderr, "Subtitle type: ");
  if (subtype == 2)
    fprintf (stderr, "SubRip (srt)\n");
  else if (subtype == 3)
    fprintf (stderr, "SubViewer (sub)\n");
  else
    fprintf (stderr, "MicroDVD (txt)\n");

  fprintf (stderr, "Output type:   ");
  if (outtype == 2)
    fprintf (stderr, "SubRip (srt)\n");
  else if (outtype == 3)
    fprintf (stderr, "SubViewer (sub)\n");
  else
    fprintf (stderr, "MicroDVD (txt)\n");

  if ((fp = fopen (argv[argc - 1], "rb")) == NULL)
    {
      perror (argv[argc - 1]);
      exit (-2);
    }

  if (!outtflag)
    outtype = subtype;
  if (ofpsflag)
    factor = fps / ofps;
  cnt = 0L;
  sub_begin = 0;
  while (!feof (fp))
    {
      if (subtype == 2)
	{
	  line = (char *) subread_srt (fp, fpout, &tbd, &ted);
	  if (line != NULL)
	    {
	      if (factor != 1.)
		{
		  if (cnt == 0L)
		    tdstart = tbd;
		  tbd = tdstart + (tbd - tdstart) * factor;
		  ted = tdstart + (ted - tdstart) * factor;
		}
	      if (move != 0.)
		{
		  tbd += move;
		  ted += move;
		}
	      tbl = tbd * fps;
	      tel = ted * fps;
	      cnt++;
	    }
	}
      else if (subtype == 3)
	{
	  line = (char *) subread_sub (fp, fpout, &tbd, &ted, &sub_begin);
	  if (line != NULL)
	    {
	      if (factor != 1.)
		{
		  if (cnt == 0L)
		    tdstart = tbd;
		  tbd = tdstart + (tbd - tdstart) * factor;
		  ted = tdstart + (ted - tdstart) * factor;
		}
	      if (move != 0.)
		{
		  tbd += move;
		  ted += move;
		}
	      tbl = tbd * fps;
	      tel = ted * fps;
	      cnt++;
	    }
	}
      else
	{
	  line = (char *) subread_txt (fp, fpout, &tbl, &tel);
	  if (line != NULL)
	    {
	      if (factor != 1.)
		{
		  if (cnt == 0L)
		    tlstart = tbl;
		  tbl = tlstart + (tbl - tlstart) * factor;
		  tel = tlstart + (tel - tlstart) * factor;
		}
	      if (move != 0.)
		{
		  tbl += move * fps;
		  tel += move * fps;
		}
	      tbd = tbl / fps;
	      ted = tel / fps;
	      cnt++;
	    }
	}
      if (line != NULL)
	{
	  if (outtype == 2)
	    {
	      /* srtprint */
	      if (cnt != 1L)
		fprintf (fpout, "\n\n");
	      fprintf (fpout, "%u\n%s --> %s\n%s", cnt, seconds2srtfmt (tbd),
		       seconds2srtfmt (ted), line);
	    }
	  else if (outtype == 3)
	    {
	      /* subprint */
	      if (cnt != 1L)
		fprintf (fpout, "\n\n");
	      fprintf (fpout, "%s,%s\n%s", seconds2subfmt (tbd),
		       seconds2subfmt (ted), line);
	    }
	  else
	    {
	      /* txtprint */
	      if (cnt != 1L)
		fprintf (fpout, "\n");
	      fprintf (fpout, "{%u}{%u}%s", tbl, tel,
		       (char *) crnl2pipe (line));
	    }
	}
    }
    fprintf (fpout, "\n");

  fclose (fp);
  fclose (fpout);

}
